/****************************************************************************
 *	Copylight(C) 2011 Kanazawa-soft-design,LLC.All Rights Reserved.
 ****************************************************************************/
/*!
 *	@file	WaveFile.cs
 *
 *	@brief	WAVEt@C̓ǂݍ.
 *
 *	@author	V 閾
 */
using System;
using System.IO;
using Microsoft.DirectX.DirectSound;

namespace WaveFile
{
	//! WAVEt@Cwb_\.
	public struct WAVFILEHEADER {
		public string	RiffId;						//!< RIFFt@Cʎq (RIFF).
		public int		FileSize;					//!< t@CTCY - 8.
		public string	FileType;					//!< t@C^Cv ("WAVE").
		public string	FormatId;					//!< tH[}bgʎq ("fmt ").
		public int		FormatSize;					//!< tH[}bgETCY - 8.
		public int		FormatType;					//!< tH[}bg^Cv.
		public short	Channels;					//!< `l.
		public int		SamplesPerSec;				//!< TvO.
		public int		AvgBytesPerSec;				//!< 1b̃oCg.
		public short	BlockAlign;					//!< 1`l̃oCg.
		public short	BitsPerSample;				//!< 1f[^̃rbg.
	}

	public class CWavFile
	{
		private System.IO.FileStream	m_Fs;		//!< WAVt@CEIuWFNg.
		private WAVFILEHEADER			m_Header;	//!< WAVt@Cwb_.

		/****************************************************************************/
		/*!
		 *	@brief	t@Cǂݍ݃[hŃI[v.
		 *
		 *	@param	[in]	file_name	t@C̃|C^.
		 *
		 *	@retval	true = OK. / false = NG.
		 */
		public bool	Open( string file_name )
		{
			//! Ƀt@CEI[v̏ꍇAG[ɂ.
			if( m_Fs != null ){
				return false;
			}

			//! sȏꍇAG[ɂ.
			if( file_name == "" ){
				return false;
			}

			try {
				//! t@Cǂݍ݃[hŃI[v.
				m_Fs = new System.IO.FileStream( file_name, System.IO.FileMode.Open, System.IO.FileAccess.Read );

				//========================================
				// t@Cwb_͂.
				//========================================

				//! t@C̐擪Xg[ǂݍ.
				byte[]	buf = new byte[256];
				int		len = Read( buf );
				if( len == 0 ){
					throw new Exception();
				}

				//! t@Cwb_o.
				LoadHeader( buf );

				//! RIFFwb_𒲂ׂ.
				if(( m_Header.RiffId   != "RIFF" )
				|| ( m_Header.FileType != "WAVE" )
				|| ( m_Header.FileSize <= 32     )){
					throw new Exception();
				}

				//! tH[}bgwb_𒲂ׂ.
				if(( m_Header.FormatId       != "fmt " )
				|| ( m_Header.Channels       ==  0     )
				|| ( m_Header.SamplesPerSec  ==  0     )
				|| ( m_Header.AvgBytesPerSec ==  0     )
				|| ( m_Header.BlockAlign     ==  0     )
				|| ( m_Header.BitsPerSample  ==  0     )
				|| ( m_Header.FormatSize     <  16     )){
					throw new Exception();
				}

				int offset = m_Header.FormatSize + 20;

				//! f[^wb_o.
				string	fcc  = GetFCC( buf, offset );
				long	size = GetDWORD( buf, offset + 4 );

				//! f[^wb_𒲂ׂ.
				if( fcc != "data" || size == 0 ){
					throw new Exception();
				}

				//! t@Cǂݍ񂾃f[^ȂꍇAG[ɂ.
				if( m_Fs.Length <= offset ){
					throw new Exception();
				}

				//! t@C̓ǂݍ݈ʒu擪t[ɍ킹Ă.
				m_Fs.Seek( offset + 8, System.IO.SeekOrigin.Begin );

				return true;
			}
			catch( Exception ){
				//! G[ꍇAt@CN[Y.
				Close();
				return false;
			}
		}

		/****************************************************************************/
		/*!
		 *	@brief	t@CEN[Y.
		 *
		 *	@param	Ȃ.
		 *
		 *	@retval	Ȃ.
		 */
		public void		Close()
		{
			//! t@CI[v̏ꍇAN[Y.
			if( m_Fs != null ){
				m_Fs.Close();
				m_Fs = null;
			}
		}

		/****************************************************************************/
		/*!
		 *	@brief	t@CWAVEf[^ǂݍ.
		 *
		 *	@param	[in]	data	WAVEf[^i[oCgf[^̔z.
		 *
		 *	@retval	t@Cǂݍ񂾃oCg.
		 */
		public int Read( byte[] data )
		{
			if( m_Fs == null ){
				return 0;
			}
			try {
				//! t@Cf[^ǂݍ.
				return m_Fs.Read( data, 0, data.Length );
			}
			catch( Exception ){
				//! G[ꍇAt@CN[Y.
				Close();
				return 0;
			}
		}

		/****************************************************************************/
		/*!
		 *	@brief	WAVt@CI[vǂ𒲂ׂ.
		 *
		 *	@param	Ȃ.
		 *
		 *	@retval	true = t@CEI[v / false = t@CEI[vĂȂ.
		 */
		public bool IsOpen()
		{
			if( m_Fs == null ){
				return false;
			}else{
				return true;
			}
		}

		/****************************************************************************/
		/*!
		 *	@brief	WAVt@C̃tH[}bg`擾.
		 *
		 *	@param	[out]	wf	WAVtH[}bg\.
		 *
		 *	@retval	Ȃ.
		 */
		public void GetWaveInfo( ref WaveFormat wf )
		{
			wf.AverageBytesPerSecond = m_Header.AvgBytesPerSec;
			wf.SamplesPerSecond      = m_Header.SamplesPerSec;
			wf.BitsPerSample         = m_Header.BitsPerSample;
			wf.BlockAlign            = m_Header.BlockAlign;
			wf.Channels              = m_Header.Channels;
			wf.FormatTag             = WaveFormatTag.Pcm;
		}

		/****************************************************************************/
		/*!
		 *	@brief	ǂݍ񂾃f[^WAVEt@C̃wb_o.
		 *
		 *	@param	[in]	data	WAVEf[^i[oCgf[^̔z.
		 *
		 *	@retval	t@Cǂݍ񂾃oCg.
		 */
		private void LoadHeader( byte[] data )
		{
			//! RIFFt@Cʎqo.
			m_Header.RiffId = GetFCC( data, 0 );

			//! t@CTCYo.
			m_Header.FileSize = GetDWORD( data, 4 );

			//! t@C^Cvo.
			m_Header.FileType = GetFCC(data, 8 );

			//! tH[}bgʎqo.
			m_Header.FormatId = GetFCC(data, 12 );

			//! tH[}bgTCYo.
			m_Header.FormatSize = GetDWORD( data, 16 );

			//! tH[}bg^Cvo.
			m_Header.FormatType = GetWORD( data, 20 );

			//! `lo.
			m_Header.Channels = GetWORD( data, 22 );

			//! TvOo.
			m_Header.SamplesPerSec = GetDWORD( data, 24 );

			//! 1b̃oCgo.
			m_Header.AvgBytesPerSec = GetDWORD( data, 28 );

			//! 1`l̃oCgo.
			m_Header.BlockAlign = GetWORD( data, 32 );

			//! 1f[^̃rbgo.
			m_Header.BitsPerSample = GetWORD( data, 34 );
		}

		/****************************************************************************/
		/*!
		 *	@brief	ǂݍ񂾃f[^16rbgo.
		 *
		 *	@param	[in]	data	Xg[f[^i[oCgf[^̔z.
		 *	@param	[in]	index	of[^̎Qƈʒu.
		 *
		 *	@retval	Xg[o4oCg̐l.
		 */
		private short GetWORD( byte[] data, int index )
		{
			int val;

			val = data[index + 1];
			val = (val * 256) + data[index];
			return (short)val;
		}

		/****************************************************************************/
		/*!
		 *	@brief	ǂݍ񂾃f[^32rbgo.
		 *
		 *	@param	[in]	data	Xg[f[^i[oCgf[^̔z.
		 *	@param	[in]	index	of[^̎Qƈʒu.
		 *
		 *	@retval	Xg[o4oCg̐l.
		 */
		private int GetDWORD( byte[] data, int index )
		{
			int val;

			val = data[index + 3];
			val = (val * 256) + data[index + 2];
			val = (val * 256) + data[index + 1];
			val = (val * 256) + data[index];
			return val;
		}

		/****************************************************************************/
		/*!
		 *	@brief	ǂݍ񂾃f[^4-CODEo.
		 *
		 *	@param	[in]	data	Xg[f[^i[oCgf[^̔z.
		 *	@param	[in]	index	of[^̎Qƈʒu.
		 *
		 *	@retval	Xg[o4-CODE̕.
		 */
		private string GetFCC( byte[] data, int index )
		{
			string str = "";

			for( int i = 0; i < 4; i++ ){
				str = str + (char)data[index + i];
			}
			return str;
		}
	}
}

/* End of File */
